@@ -1440,6 +1440,15 @@ gcm_session_add_x11_output (GsdColorManager *manager, GnomeRROutput *output)
                              gnome_rr_output_get_is_primary (output) ?
                              (gpointer) CD_DEVICE_METADATA_OUTPUT_PRIORITY_PRIMARY :
                              (gpointer) CD_DEVICE_METADATA_OUTPUT_PRIORITY_SECONDARY);
+#endif
+#if CD_CHECK_VERSION(0,1,27)
+        /* set this so we can call the device a 'Laptop Screen' in the
+         * control center main panel */
+        if (gnome_rr_output_is_laptop (output)) {
+                g_hash_table_insert (device_props,
+                                     (gpointer) CD_DEVICE_PROPERTY_EMBEDDED,
+                                     NULL);
+        }
 #endif
         cd_client_create_device (priv->client,
                                  device_id,
