@@ -53,12 +53,14 @@
 
 #include "shortcuts-list.h"
 #include "gsd-osd-window.h"
+#include "gsd-screenshot-utils.h"
 #include "gsd-input-helper.h"
 #include "gsd-enums.h"
 
 #include <canberra.h>
 #include <pulse/pulseaudio.h>
 #include "gvc-mixer-control.h"
+#include "gvc-mixer-sink.h"
 
 #define GSD_MEDIA_KEYS_DBUS_PATH GSD_DBUS_PATH "/MediaKeys"
 #define GSD_MEDIA_KEYS_DBUS_NAME GSD_DBUS_NAME ".MediaKeys"
@@ -123,7 +125,8 @@ struct GsdMediaKeysManagerPrivate
 {
         /* Volume bits */
         GvcMixerControl *volume;
-        GvcMixerStream  *stream;
+        GvcMixerStream  *sink;
+        GvcMixerStream  *source;
         ca_context      *ca;
         GtkSettings     *gtksettings;
 #ifdef HAVE_GUDEV
@@ -144,6 +147,7 @@ struct GsdMediaKeysManagerPrivate
 
         /* Power stuff */
         GSettings       *power_settings;
+        GDBusProxy      *power_proxy;
         GDBusProxy      *power_screen_proxy;
         GDBusProxy      *power_keyboard_proxy;
 
@@ -828,9 +832,11 @@ do_eject_action (GsdMediaKeysManager *manager)
 
         /* Show the dialogue */
         dialog_init (manager);
+        gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
+                                         NULL);
         gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                 "media-eject-symbolic",
-                                                 FALSE);
+                                          "media-eject-symbolic",
+                                          FALSE);
         dialog_show (manager);
 
         /* Clean up the drive selection and exit if no suitable
@@ -870,22 +876,6 @@ do_home_key_action (GsdMediaKeysManager *manager,
 	g_free (uri);
 }
 
-static void
-do_execute_desktop (GsdMediaKeysManager *manager,
-		    const char          *desktop,
-		    gint64               timestamp)
-{
-        GDesktopAppInfo *app_info;
-
-        app_info = g_desktop_app_info_new (desktop);
-        if (app_info != NULL) {
-                launch_app (G_APP_INFO (app_info), timestamp);
-                g_object_unref (app_info);
-        } else {
-                g_warning ("Could not find application '%s'", desktop);
-	}
-}
-
 static void
 do_execute_desktop_or_desktop (GsdMediaKeysManager *manager,
 			       const char          *desktop,
@@ -911,9 +901,11 @@ static void
 do_touchpad_osd_action (GsdMediaKeysManager *manager, gboolean state)
 {
         dialog_init (manager);
+        gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
+                                         NULL);
         gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                 state ? "input-touchpad-symbolic" : "touchpad-disabled-symbolic",
-                                                 FALSE);
+                                          state ? "input-touchpad-symbolic" : "touchpad-disabled-symbolic",
+                                          FALSE);
         dialog_show (manager);
 }
 
@@ -945,6 +937,10 @@ update_dialog (GsdMediaKeysManager *manager,
                gboolean             sound_changed,
                gboolean             quiet)
 {
+        GvcMixerUIDevice *device;
+        const GvcMixerStreamPort *port;
+        GsdOsdWindowAction action;
+
         if (!muted) {
                 vol = (int) (100 * (double) vol / PA_VOLUME_NORM);
                 vol = CLAMP (vol, 0, 100);
@@ -952,12 +948,21 @@ update_dialog (GsdMediaKeysManager *manager,
                 vol = 0.0;
         }
 
+        action = GVC_IS_MIXER_SINK (stream) ? GSD_OSD_WINDOW_ACTION_VOLUME : GSD_OSD_WINDOW_ACTION_MIC_VOLUME;
+
         dialog_init (manager);
-        gsd_osd_window_set_volume_muted (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                muted);
+        gsd_osd_window_set_volume_muted (GSD_OSD_WINDOW (manager->priv->dialog), muted);
         gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (manager->priv->dialog), vol);
-        gsd_osd_window_set_action (GSD_OSD_WINDOW (manager->priv->dialog),
-                                          GSD_OSD_WINDOW_ACTION_VOLUME);
+        gsd_osd_window_set_action (GSD_OSD_WINDOW (manager->priv->dialog), action);
+
+        port = gvc_mixer_stream_get_port (stream);
+        if (g_strcmp0 (gvc_mixer_stream_get_form_factor (stream), "internal") != 0 ||
+            g_strcmp0 (port->port, "analog-output-speaker") != 0) {
+                device = gvc_mixer_control_lookup_device_from_stream (manager->priv->volume, stream);
+                gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
+                                                 gvc_mixer_ui_device_get_description (device));
+        }
+
         dialog_show (manager);
 
         if (quiet == FALSE && sound_changed != FALSE && muted == FALSE) {
@@ -990,13 +995,14 @@ get_udev_device_for_sysfs_path (GsdMediaKeysManager *manager,
 
 static GvcMixerStream *
 get_stream_for_device_id (GsdMediaKeysManager *manager,
+			  gboolean             is_output,
 			  guint                deviceid)
 {
 	char *devnode;
 	gpointer id_ptr;
 	GvcMixerStream *res;
 	GUdevDevice *dev, *parent;
-	GSList *sinks, *l;
+	GSList *streams, *l;
 
 	id_ptr = g_hash_table_lookup (manager->priv->streams, GUINT_TO_POINTER (deviceid));
 	if (id_ptr != NULL) {
@@ -1037,30 +1043,35 @@ get_stream_for_device_id (GsdMediaKeysManager *manager,
 	}
 
 	res = NULL;
-	sinks = gvc_mixer_control_get_sinks (manager->priv->volume);
-	for (l = sinks; l; l = l->next) {
+	if (is_output)
+		streams = gvc_mixer_control_get_sinks (manager->priv->volume);
+	else
+		streams = gvc_mixer_control_get_sources (manager->priv->volume);
+	for (l = streams; l; l = l->next) {
 		GvcMixerStream *stream = l->data;
 		const char *sysfs_path;
-		GUdevDevice *sink_dev, *sink_parent;
+		GUdevDevice *stream_dev, *stream_parent;
 
 		sysfs_path = gvc_mixer_stream_get_sysfs_path (stream);
-		sink_dev = get_udev_device_for_sysfs_path (manager, sysfs_path);
-		if (sink_dev == NULL)
+		stream_dev = get_udev_device_for_sysfs_path (manager, sysfs_path);
+		if (stream_dev == NULL)
 			continue;
-		sink_parent = g_udev_device_get_parent_with_subsystem (sink_dev, "usb", "usb_device");
-		g_object_unref (sink_dev);
-		if (sink_parent == NULL)
+		stream_parent = g_udev_device_get_parent_with_subsystem (stream_dev, "usb", "usb_device");
+		g_object_unref (stream_dev);
+		if (stream_parent == NULL)
 			continue;
 
-		if (g_strcmp0 (g_udev_device_get_sysfs_path (sink_parent),
+		if (g_strcmp0 (g_udev_device_get_sysfs_path (stream_parent),
 			       g_udev_device_get_sysfs_path (parent)) == 0) {
 			res = stream;
 		}
-		g_object_unref (sink_parent);
+		g_object_unref (stream_parent);
 		if (res != NULL)
 			break;
 	}
 
+	g_slist_free (streams);
+
 	if (res)
 		g_hash_table_insert (manager->priv->streams,
 				     GUINT_TO_POINTER (deviceid),
@@ -1078,6 +1089,7 @@ static void
 do_sound_action (GsdMediaKeysManager *manager,
 		 guint                deviceid,
                  int                  type,
+                 gboolean             is_output,
                  gboolean             quiet)
 {
 	GvcMixerStream *stream;
@@ -1086,11 +1098,18 @@ do_sound_action (GsdMediaKeysManager *manager,
         gboolean sound_changed;
 
         /* Find the stream that corresponds to the device, if any */
+        stream = NULL;
 #ifdef HAVE_GUDEV
-        stream = get_stream_for_device_id (manager, deviceid);
-        if (stream == NULL)
+        stream = get_stream_for_device_id (manager, is_output, deviceid);
 #endif /* HAVE_GUDEV */
-                stream = manager->priv->stream;
+
+        if (stream == NULL) {
+                if (is_output)
+                        stream = manager->priv->sink;
+                else
+                        stream = manager->priv->source;
+        }
+
         if (stream == NULL)
                 return;
 
@@ -1155,27 +1174,44 @@ update_default_sink (GsdMediaKeysManager *manager)
         GvcMixerStream *stream;
 
         stream = gvc_mixer_control_get_default_sink (manager->priv->volume);
-        if (stream == manager->priv->stream)
+        if (stream == manager->priv->sink)
                 return;
 
-        if (manager->priv->stream != NULL) {
-                g_object_unref (manager->priv->stream);
-                manager->priv->stream = NULL;
-        }
+        g_clear_object (&manager->priv->sink);
 
         if (stream != NULL) {
-                manager->priv->stream = g_object_ref (stream);
+                manager->priv->sink = g_object_ref (stream);
         } else {
                 g_warning ("Unable to get default sink");
         }
 }
 
+static void
+update_default_source (GsdMediaKeysManager *manager)
+{
+        GvcMixerStream *stream;
+
+        stream = gvc_mixer_control_get_default_source (manager->priv->volume);
+        if (stream == manager->priv->source)
+                return;
+
+        g_clear_object (&manager->priv->source);
+
+        if (stream != NULL) {
+                manager->priv->source = g_object_ref (stream);
+                g_message ("update_default_source done ");
+        } else {
+                g_warning ("Unable to get default source");
+        }
+}
+
 static void
 on_control_state_changed (GvcMixerControl     *control,
                           GvcMixerControlState new_state,
                           GsdMediaKeysManager *manager)
 {
         update_default_sink (manager);
+        update_default_source (manager);
 }
 
 static void
@@ -1186,6 +1222,14 @@ on_control_default_sink_changed (GvcMixerControl     *control,
         update_default_sink (manager);
 }
 
+static void
+on_control_default_source_changed (GvcMixerControl     *control,
+                                   guint                id,
+                                   GsdMediaKeysManager *manager)
+{
+        update_default_source (manager);
+}
+
 #ifdef HAVE_GUDEV
 static gboolean
 remove_stream (gpointer key,
@@ -1203,11 +1247,13 @@ on_control_stream_removed (GvcMixerControl     *control,
                            guint                id,
                            GsdMediaKeysManager *manager)
 {
-        if (manager->priv->stream != NULL) {
-		if (gvc_mixer_stream_get_id (manager->priv->stream) == id) {
-	                g_object_unref (manager->priv->stream);
-			manager->priv->stream = NULL;
-		}
+        if (manager->priv->sink != NULL) {
+		if (gvc_mixer_stream_get_id (manager->priv->sink) == id)
+			g_clear_object (&manager->priv->sink);
+        }
+        if (manager->priv->source != NULL) {
+		if (gvc_mixer_stream_get_id (manager->priv->source) == id)
+			g_clear_object (&manager->priv->source);
         }
 
 #ifdef HAVE_GUDEV
@@ -1376,9 +1422,11 @@ gsd_media_player_key_pressed (GsdMediaKeysManager *manager,
         if (!have_listeners) {
                 /* Popup a dialog with an (/) icon */
                 dialog_init (manager);
+                gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
+                                                 NULL);
                 gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                         "action-unavailable-symbolic",
-                                                         FALSE);
+                                                  "action-unavailable-symbolic",
+                                                  FALSE);
                 dialog_show (manager);
                 return TRUE;
         }
@@ -1703,9 +1751,11 @@ update_screen_cb (GObject             *source_object,
         /* update the dialog with the new value */
         g_variant_get (new_percentage, "(u)", &percentage);
         dialog_init (manager);
+        gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
+                                         NULL);
         gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                 "display-brightness-symbolic",
-                                                 TRUE);
+                                          "display-brightness-symbolic",
+                                          TRUE);
         gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (manager->priv->dialog),
                                                 percentage);
         dialog_show (manager);
@@ -1755,9 +1805,11 @@ update_keyboard_cb (GObject             *source_object,
         /* update the dialog with the new value */
         g_variant_get (new_percentage, "(u)", &percentage);
         dialog_init (manager);
+        gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
+                                         NULL);
         gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                 "keyboard-brightness-symbolic",
-                                                 TRUE);
+                                          "keyboard-brightness-symbolic",
+                                          TRUE);
         gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (manager->priv->dialog),
                                                 percentage);
         dialog_show (manager);
@@ -1801,6 +1853,38 @@ do_keyboard_brightness_action (GsdMediaKeysManager *manager,
                            manager);
 }
 
+static void
+do_battery_action (GsdMediaKeysManager *manager)
+{
+        GVariant *icon_var, *percentage;
+        GIcon *icon;
+        gboolean show_percentage = FALSE;
+
+        if (manager->priv->power_proxy == NULL)
+                return;
+
+        icon_var = g_dbus_proxy_get_cached_property (manager->priv->power_proxy, "Icon");
+        percentage = g_dbus_proxy_get_cached_property (manager->priv->power_proxy, "Percentage");
+
+        dialog_init (manager);
+        if (g_variant_get_double (percentage) >= 0.0) {
+                char *str;
+
+                show_percentage = TRUE;
+                str = g_strdup_printf ("%d %%", (int) g_variant_get_double (percentage));
+                gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (manager->priv->dialog),
+                                                 g_variant_get_double (percentage));
+                gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
+                                                 str);
+                g_free (str);
+        }
+        icon = g_icon_new_for_string (g_variant_get_string (icon_var, NULL), NULL);
+        gsd_osd_window_set_action_custom_gicon (GSD_OSD_WINDOW (manager->priv->dialog),
+                                                icon,
+                                                show_percentage);
+        dialog_show (manager);
+}
+
 static void
 do_custom_action (GsdMediaKeysManager *manager,
                   MediaKey            *key,
@@ -1830,16 +1914,19 @@ do_action (GsdMediaKeysManager *manager,
         case MUTE_KEY:
         case VOLUME_DOWN_KEY:
         case VOLUME_UP_KEY:
-                do_sound_action (manager, deviceid, type, FALSE);
+                do_sound_action (manager, deviceid, type, TRUE, FALSE);
+                break;
+        case MIC_MUTE_KEY:
+                do_sound_action (manager, deviceid, MUTE_KEY, FALSE, TRUE);
                 break;
         case MUTE_QUIET_KEY:
-                do_sound_action (manager, deviceid, MUTE_KEY, TRUE);
+                do_sound_action (manager, deviceid, MUTE_KEY, TRUE, TRUE);
                 break;
         case VOLUME_DOWN_QUIET_KEY:
-                do_sound_action (manager, deviceid, VOLUME_DOWN_KEY, TRUE);
+                do_sound_action (manager, deviceid, VOLUME_DOWN_KEY, TRUE, TRUE);
                 break;
         case VOLUME_UP_QUIET_KEY:
-                do_sound_action (manager, deviceid, VOLUME_UP_KEY, TRUE);
+                do_sound_action (manager, deviceid, VOLUME_UP_KEY, TRUE, TRUE);
                 break;
         case LOGOUT_KEY:
                 gnome_session_shutdown (manager);
@@ -1866,22 +1953,12 @@ do_action (GsdMediaKeysManager *manager,
                 do_url_action (manager, "ghelp", timestamp);
                 break;
         case SCREENSHOT_KEY:
-                execute (manager, "gnome-screenshot", FALSE);
-                break;
-        case WINDOW_SCREENSHOT_KEY:
-                execute (manager, "gnome-screenshot --window", FALSE);
-                break;
-        case AREA_SCREENSHOT_KEY:
-                execute (manager, "gnome-screenshot --area", FALSE);
-                break;
         case SCREENSHOT_CLIP_KEY:
-                execute (manager, "gnome-screenshot --clipboard", FALSE);
-                break;
+        case WINDOW_SCREENSHOT_KEY:
         case WINDOW_SCREENSHOT_CLIP_KEY:
-                execute (manager, "gnome-screenshot --window --clipboard", FALSE);
-                break;
+        case AREA_SCREENSHOT_KEY:
         case AREA_SCREENSHOT_CLIP_KEY:
-                execute (manager, "gnome-screenshot --area --clipboard", FALSE);
+                gsd_screenshot_take (type);
                 break;
         case WWW_KEY:
                 do_url_action (manager, "http", timestamp);
@@ -1958,7 +2035,7 @@ do_action (GsdMediaKeysManager *manager,
                 do_keyboard_brightness_action (manager, type);
                 break;
         case BATTERY_KEY:
-                do_execute_desktop (manager, "gnome-power-statistics.desktop", timestamp);
+                do_battery_action (manager);
                 break;
         /* Note, no default so compiler catches missing keys */
         case CUSTOM_KEY:
@@ -2078,6 +2155,41 @@ update_theme_settings (GSettings           *settings,
 	}
 }
 
+static void
+initialize_volume_handler (GsdMediaKeysManager *manager)
+{
+        /* initialise Volume handler
+         *
+         * We do this one here to force checking gstreamer cache, etc.
+         * The rest (grabbing and setting the keys) can happen in an
+         * idle.
+         */
+        gnome_settings_profile_start ("gvc_mixer_control_new");
+
+        manager->priv->volume = gvc_mixer_control_new ("GNOME Volume Control Media Keys");
+
+        g_signal_connect (manager->priv->volume,
+                          "state-changed",
+                          G_CALLBACK (on_control_state_changed),
+                          manager);
+        g_signal_connect (manager->priv->volume,
+                          "default-sink-changed",
+                          G_CALLBACK (on_control_default_sink_changed),
+                          manager);
+        g_signal_connect (manager->priv->volume,
+                          "default-source-changed",
+                          G_CALLBACK (on_control_default_source_changed),
+                          manager);
+        g_signal_connect (manager->priv->volume,
+                          "stream-removed",
+                          G_CALLBACK (on_control_stream_removed),
+                          manager);
+
+        gvc_mixer_control_open (manager->priv->volume);
+
+        gnome_settings_profile_end ("gvc_mixer_control_new");
+}
+
 static gboolean
 start_media_keys_idle_cb (GsdMediaKeysManager *manager)
 {
@@ -2087,6 +2199,8 @@ start_media_keys_idle_cb (GsdMediaKeysManager *manager)
         g_debug ("Starting media_keys manager");
         gnome_settings_profile_start (NULL);
 
+        initialize_volume_handler (manager);
+
         manager->priv->settings = g_settings_new (SETTINGS_BINDING_DIR);
         g_signal_connect (G_OBJECT (manager->priv->settings), "changed",
                           G_CALLBACK (gsettings_changed_cb), manager);
@@ -2168,33 +2282,6 @@ gsd_media_keys_manager_start (GsdMediaKeysManager *manager,
         manager->priv->udev_client = g_udev_client_new (subsystems);
 #endif
 
-        /* initialise Volume handler
-         *
-         * We do this one here to force checking gstreamer cache, etc.
-         * The rest (grabbing and setting the keys) can happen in an
-         * idle.
-         */
-        gnome_settings_profile_start ("gvc_mixer_control_new");
-
-        manager->priv->volume = gvc_mixer_control_new ("GNOME Volume Control Media Keys");
-
-        g_signal_connect (manager->priv->volume,
-                          "state-changed",
-                          G_CALLBACK (on_control_state_changed),
-                          manager);
-        g_signal_connect (manager->priv->volume,
-                          "default-sink-changed",
-                          G_CALLBACK (on_control_default_sink_changed),
-                          manager);
-        g_signal_connect (manager->priv->volume,
-                          "stream-removed",
-                          G_CALLBACK (on_control_stream_removed),
-                          manager);
-
-        gvc_mixer_control_open (manager->priv->volume);
-
-        gnome_settings_profile_end ("gvc_mixer_control_new");
-
         manager->priv->start_idle_id = g_idle_add ((GSourceFunc) start_media_keys_idle_cb, manager);
 
         register_manager (manager_object);
@@ -2209,7 +2296,6 @@ gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
 {
         GsdMediaKeysManagerPrivate *priv = manager->priv;
         GSList *ls;
-        GList *l;
         int i;
 
         g_debug ("Stopping media_keys manager");
@@ -2231,58 +2317,27 @@ gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
                 manager->priv->gtksettings = NULL;
         }
 
-        if (manager->priv->ca) {
-                ca_context_destroy (manager->priv->ca);
-                manager->priv->ca = NULL;
-        }
+        g_clear_pointer (&manager->priv->ca, ca_context_destroy);
 
 #ifdef HAVE_GUDEV
-        if (priv->streams) {
-                g_hash_table_destroy (priv->streams);
-                priv->streams = NULL;
-        }
-        if (priv->udev_client) {
-                g_object_unref (priv->udev_client);
-                priv->udev_client = NULL;
-        }
+        g_clear_pointer (&priv->streams, g_hash_table_destroy);
+        g_clear_object (&priv->udev_client);
 #endif /* HAVE_GUDEV */
 
         g_clear_object (&priv->logind_proxy);
-        if (priv->settings) {
-                g_object_unref (priv->settings);
-                priv->settings = NULL;
-        }
-
-        if (priv->power_settings) {
-                g_object_unref (priv->power_settings);
-                priv->power_settings = NULL;
-        }
-
-        if (priv->power_screen_proxy) {
-                g_object_unref (priv->power_screen_proxy);
-                priv->power_screen_proxy = NULL;
-        }
-
-        if (priv->power_keyboard_proxy) {
-                g_object_unref (priv->power_keyboard_proxy);
-                priv->power_keyboard_proxy = NULL;
-        }
+        g_clear_object (&priv->settings);
+        g_clear_object (&priv->power_settings);
+        g_clear_object (&priv->power_proxy);
+        g_clear_object (&priv->power_screen_proxy);
+        g_clear_object (&priv->power_keyboard_proxy);
 
         if (priv->cancellable != NULL) {
                 g_cancellable_cancel (priv->cancellable);
-                g_object_unref (priv->cancellable);
-                priv->cancellable = NULL;
+                g_clear_object (&priv->cancellable);
         }
 
-        if (priv->introspection_data) {
-                g_dbus_node_info_unref (priv->introspection_data);
-                priv->introspection_data = NULL;
-        }
-
-        if (priv->connection != NULL) {
-                g_object_unref (priv->connection);
-                priv->connection = NULL;
-        }
+        g_clear_pointer (&priv->introspection_data, g_dbus_node_info_unref);
+        g_clear_object (&priv->connection);
 
         if (priv->keys != NULL) {
                 gdk_error_trap_push ();
@@ -2301,33 +2356,14 @@ gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
                 gdk_error_trap_pop_ignored ();
         }
 
-        if (priv->screens != NULL) {
-                g_slist_free (priv->screens);
-                priv->screens = NULL;
-        }
-
-        if (priv->stream) {
-                g_object_unref (priv->stream);
-                priv->stream = NULL;
-        }
-
-        if (priv->volume) {
-                g_object_unref (priv->volume);
-                priv->volume = NULL;
-        }
-
-        if (priv->dialog != NULL) {
-                gtk_widget_destroy (priv->dialog);
-                priv->dialog = NULL;
-        }
+        g_clear_pointer (&priv->screens, g_slist_free);
+        g_clear_object (&priv->sink);
+        g_clear_object (&priv->source);
+        g_clear_object (&priv->volume);
+        g_clear_object (&priv->dialog);
 
         if (priv->media_players != NULL) {
-                for (l = priv->media_players; l; l = l->next) {
-                        MediaPlayer *mp = l->data;
-                        g_free (mp->application);
-                        g_free (mp);
-                }
-                g_list_free (priv->media_players);
+                g_list_free_full (priv->media_players, (GDestroyNotify) free_media_player);
                 priv->media_players = NULL;
         }
 }
@@ -2473,6 +2509,21 @@ xrandr_ready_cb (GObject             *source_object,
         }
 }
 
+static void
+power_ready_cb (GObject             *source_object,
+                GAsyncResult        *res,
+                GsdMediaKeysManager *manager)
+{
+        GError *error = NULL;
+
+        manager->priv->power_proxy = g_dbus_proxy_new_finish (res, &error);
+        if (manager->priv->power_proxy == NULL) {
+                g_warning ("Failed to get proxy for power: %s",
+                           error->message);
+                g_error_free (error);
+        }
+}
+
 static void
 power_screen_ready_cb (GObject             *source_object,
                        GAsyncResult        *res,
@@ -2543,6 +2594,16 @@ on_bus_gotten (GObject             *source_object,
                           (GAsyncReadyCallback) xrandr_ready_cb,
                           manager);
 
+        g_dbus_proxy_new (manager->priv->connection,
+                          G_DBUS_PROXY_FLAGS_NONE,
+                          NULL,
+                          GSD_DBUS_NAME ".Power",
+                          GSD_DBUS_PATH "/Power",
+                          GSD_DBUS_BASE_INTERFACE ".Power",
+                          NULL,
+                          (GAsyncReadyCallback) power_ready_cb,
+                          manager);
+
         g_dbus_proxy_new (manager->priv->connection,
                           G_DBUS_PROXY_FLAGS_NONE,
                           NULL,
