@@ -1689,15 +1689,47 @@ out:
         g_object_unref (current);
 }
 
-static void
-auto_configure_outputs (GsdXrandrManager *manager, guint32 timestamp)
+static GnomeRRConfig *
+make_default_setup (GsdXrandrManager *manager)
 {
         GsdXrandrManagerPrivate *priv = manager->priv;
         GnomeRRConfig *config;
+        GsdXrandrBootBehaviour boot;
 
-        config = make_xinerama_setup (manager, priv->rw_screen);
+        boot = g_settings_get_enum (priv->settings, CONF_KEY_DEFAULT_MONITORS_SETUP);
+        g_debug ("xrandr default monitors setup: %d\n", boot);
+
+        switch (boot) {
+        case GSD_XRANDR_BOOT_BEHAVIOUR_DO_NOTHING:
+                config = make_xinerama_setup (manager, priv->rw_screen);
+                break;
+        case GSD_XRANDR_BOOT_BEHAVIOUR_FOLLOW_LID:
+                if (laptop_lid_is_closed (manager))
+                        config = make_other_setup (priv->rw_screen);
+                else
+                        config = make_xinerama_setup (manager, priv->rw_screen);
+                break;
+        case GSD_XRANDR_BOOT_BEHAVIOUR_CLONE:
+                config = make_clone_setup (manager, priv->rw_screen);
+                break;
+        case GSD_XRANDR_BOOT_BEHAVIOUR_DOCK:
+                config = make_other_setup (priv->rw_screen);
+                break;
+        default:
+                g_assert_not_reached ();
+        }
+
+        return config;
+}
+
+static void
+auto_configure_outputs (GsdXrandrManager *manager, guint32 timestamp)
+{
+        GnomeRRConfig *config;
+
+        g_debug ("xrandr auto-configure\n");
+        config = make_default_setup (manager);
         if (config) {
-                print_configuration (config, "auto-configure - xinerama mode");
                 apply_configuration (manager, config, timestamp, TRUE, FALSE);
                 g_object_unref (config);
         } else {
@@ -1868,31 +1900,15 @@ static void
 apply_default_boot_configuration (GsdXrandrManager *mgr, guint32 timestamp)
 {
         GsdXrandrManagerPrivate *priv = mgr->priv;
-        GnomeRRScreen *screen = priv->rw_screen;
         GnomeRRConfig *config;
         GsdXrandrBootBehaviour boot;
 
         boot = g_settings_get_enum (priv->settings, CONF_KEY_DEFAULT_MONITORS_SETUP);
 
-        switch (boot) {
-        case GSD_XRANDR_BOOT_BEHAVIOUR_DO_NOTHING:
+        if (boot == GSD_XRANDR_BOOT_BEHAVIOUR_DO_NOTHING)
                 return;
-        case GSD_XRANDR_BOOT_BEHAVIOUR_FOLLOW_LID:
-                if (laptop_lid_is_closed (mgr))
-                        config = make_other_setup (screen);
-                else
-                        config = make_xinerama_setup (mgr, screen);
-                break;
-        case GSD_XRANDR_BOOT_BEHAVIOUR_CLONE:
-                config = make_clone_setup (mgr, screen);
-                break;
-        case GSD_XRANDR_BOOT_BEHAVIOUR_DOCK:
-                config = make_other_setup (screen);
-                break;
-        default:
-                g_assert_not_reached ();
-        }
 
+        config = make_default_setup (mgr);
         if (config) {
                 /* We don't save the configuration (the "false" parameter to the following function) because we don't want to
                  * install a user-side setting when *here* we are using a system-default setting.
@@ -1911,6 +1927,8 @@ apply_stored_configuration_at_startup (GsdXrandrManager *manager, guint32 timest
         char *intended_filename;
         GnomePnpIds *pnp_ids;
 
+        /* This avoids the GnomePnpIds object being created multiple times.
+         * See c9240e8b69c5833074508b46bc56307aac12ec19 */
         pnp_ids = gnome_pnp_ids_new ();
         backup_filename = gnome_rr_config_get_backup_filename ();
         intended_filename = gnome_rr_config_get_intended_filename ();
